@@ -4,7 +4,9 @@ use strict;
 use base 'Exporter';
 use Dpkg::IPC;
 use File::Path qw(make_path);
+use Graph;
 use JSON;
+use Debian::PkgJs::Npm;
 
 use constant CLEANLIST => [
     qw(
@@ -32,6 +34,7 @@ our @EXPORT = qw(
   normalize_name_strict
   open_file
   main_package
+  packages_list
   pjson
   set_debug
   search_bd_in_debian_control
@@ -61,6 +64,7 @@ sub open_file {
     my $f;
     open( $f, $file ) or die $!;
     my @lines = map {
+        s/\r//g;
         chomp;
         s/#.*$//;
         s/^\s*(.*?)\s*/$1/;
@@ -118,7 +122,6 @@ sub readLerna {
     my $dir = pop;
     return $lernaFiles{$dir} if $lernaFiles{$dir};
     my $globs = {};
-    my $done;
     if ( -e "$dir/lerna.json" ) {
         debug("$dir/lerna.json found\n");
         my $jsonc;
@@ -126,11 +129,23 @@ sub readLerna {
         my $content = join '', <$jsonc>;
         close $jsonc;
         $globs = JSON::from_json($content);
-        $done++ unless ( $globs->{useWorkspaces} );
     }
-    if ( !$done and my $tmp = pjson($dir)->{workspaces} ) {
+    if ( my $tmp = pjson($dir)->{workspaces} ) {
         debug("workspaces field found in package.json\n");
-        $globs->{packages} = $tmp;
+        if ( ref $tmp ) {
+            if ( ref($tmp) eq 'HASH' and $tmp->{packages} ) {
+                $globs->{packages} = $tmp->{packages};
+            }
+            elsif ( ref $tmp eq 'ARRAY' ) {
+                $globs->{packages} = $tmp;
+            }
+            else {
+                print STDERR "Unsupported $dir/package.json format\n";
+            }
+        }
+        else {
+            print STDERR "Bad $dir/package.json#workspaces\n";
+        }
     }
     $lernaFiles{$dir} = $globs;
     if ( my $lpkgs = $globs->{packages} ) {
@@ -623,14 +638,82 @@ sub nodepath {
     return $path;
 }
 
-sub search_bd_in_debian_control {
-    my $re = pop;
+sub _debian_control {
     require Debian::Control;
     my $c = Debian::Control->new;
     $c->read('debian/control');
+    return $c;
+}
+
+sub search_bd_in_debian_control {
+    my $re    = pop;
     my $found = 0;
-    map { $found++ if $_->pkg =~ $re } @{ $c->source->Build_Depends };
+    map { $found++ if $_->pkg =~ $re }
+      @{ _debian_control->source->Build_Depends };
     return $found;
 }
 
+sub packages_list {
+    return keys %{ _debian_control->binary };
+}
+
+sub ordered_components_list {
+    my $components = component_list;
+    my @cmps       = sort keys %$components;
+    $components->{&main_package} = pjson(main_package)->{name};
+    push @cmps, &main_package;
+    my $modules = { map { ( $components->{$_} => $_ ) } keys %$components };
+
+    #my $g = Graph->new;
+    #$g->add_vertices( keys %$components );
+    my %needs;
+
+    sub deplist {
+        my ($cmp) = @_;
+        return () unless -d $cmp;
+        my $name = pjson($cmp)->{name};
+        my @deps = sort ( ( keys %{ pjson($cmp)->{dependencies} // {} } ),
+            ( keys %{ pjson($cmp)->{devDependencies}  // {} } ),
+            ( keys %{ pjson($cmp)->{peerDependencies} // {} } ),
+        );
+        map {
+            my $s = $components->{$_};
+
+          #map { $g->add_edge( $modules->{$_}, $cmp ) } grep { $_ eq $s } @deps;
+            map { $needs{$cmp}->{ $modules->{$_} } = 1 }
+              grep { $_ eq $s } @deps;
+        } @cmps;
+    }
+    deplist($_) foreach (@cmps);
+    my ( @list, @res );
+    foreach my $cmp (@cmps) {
+        if ( $needs{$cmp} ) {
+            push @list, $cmp;
+        }
+        else {
+            push @res, $cmp;
+        }
+    }
+
+    sub findB {
+        my ( $b, $deps, $level ) = @_;
+        return 1 if $deps->{$b};
+        return 0 if $level >= $#list;
+        foreach my $k ( keys %$deps ) {
+            return 1 if findB( $b, $needs{$k}, $level + 1 );
+        }
+        return 0;
+    }
+    push @res, sort {
+        return 1  if findB( $b, $needs{$a}, 0 );
+        return -1 if findB( $a, $needs{$b}, 0 );
+    } @list;
+
+    #eval { @res = $g->toposort };
+    #if ($@) {
+    #    die "Unable to resolve automatically components build order"
+    #}
+    return @res;
+}
+
 1;
